/*
 * Copyright 2019 Oliwia Chrapowicka
 *
 * This file is part of QESG.
 *
 * QESG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * QESG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with QESG.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "add_atomic_pos.h"

using namespace std;


add_atomic_pos_window::add_atomic_pos_window()
{
    set_title("Add atomic positions");
    add(*main_box);

    box_for_infos->set_orientation(Gtk::ORIENTATION_HORIZONTAL);
	box_for_infos->set_border_width(60);
	box_for_infos->set_spacing(5);
	box_for_infos->set_homogeneous();

    entry_atom_name->set_editable(false);
    entry_atom_num->set_editable(false);

    box_for_infos->add(*label_entry_atom_name);
    box_for_infos->add(*entry_atom_name);
    box_for_infos->add(*label_entry_atom_num);
    box_for_infos->add(*entry_atom_num);

    box_for_entries->set_orientation(Gtk::ORIENTATION_HORIZONTAL);
	box_for_entries->set_border_width(10);
	box_for_entries->set_spacing(10);
	box_for_entries->set_homogeneous();

	buttons_pos1.set_adjustment(adj_pos1);
	buttons_pos1.set_snap_to_ticks();
	buttons_pos1.set_numeric();
	buttons_pos1.set_digits(6);

    buttons_pos2.set_adjustment(adj_pos2);
    buttons_pos2.set_snap_to_ticks();
    buttons_pos2.set_numeric();
    buttons_pos2.set_digits(6);

    buttons_pos3.set_adjustment(adj_pos3);
    buttons_pos3.set_snap_to_ticks();
    buttons_pos3.set_numeric();
    buttons_pos3.set_digits(6);

    buttons_1.set_adjustment(adj_1);
    buttons_1.set_snap_to_ticks();
    buttons_1.set_numeric();
    buttons_1.set_digits(0);

    buttons_2.set_adjustment(adj_2);
    buttons_2.set_snap_to_ticks();
    buttons_2.set_numeric();
    buttons_2.set_digits(0);

    buttons_3.set_adjustment(adj_3);
    buttons_3.set_snap_to_ticks();
    buttons_3.set_numeric();
    buttons_3.set_digits(0);

	box_for_entries->add(buttons_pos1);
    box_for_entries->add(buttons_pos2);
    box_for_entries->add(buttons_pos3);
    box_for_entries->add(buttons_1);
    box_for_entries->add(buttons_2);
    box_for_entries->add(buttons_3);

    box_for_button->set_orientation(Gtk::ORIENTATION_HORIZONTAL);
    box_for_button->set_border_width(60);
    box_for_button->set_spacing(5);

    button_add_atom_pos.set_label("Add atom's position");
    button_add_atom_pos.signal_clicked().connect(sigc::mem_fun(*this, &add_atomic_pos_window::add_one_atom));

    box_for_button->pack_end(button_add_atom_pos);

    main_box->add(*label_info);
    main_box->add(*box_for_infos);
    main_box->add(*box_for_entries);
    main_box->add(*box_for_button);

    main_box->show_all(); //necessary to show items in a mbox
}

add_atomic_pos_window::~add_atomic_pos_window()
{
    //deconstructor
}

void add_atomic_pos_window:: set_params(string name, int num)
{
    name_of_atom = name;
    num_left = num;
    entry_atom_name->set_text(name_of_atom);
    entry_atom_num->set_text(to_string(num_left));
}

void add_atomic_pos_window:: add_one_atom()
{
    Glib::ustring at_pos = name_of_atom + " " + to_string(buttons_pos1.get_value()) + " " + to_string(buttons_pos2.get_value()) + " " + to_string(buttons_pos3.get_value()) + " " + to_string(int(buttons_1.get_value())) + " " + to_string(int(buttons_2.get_value())) + " " + to_string(int(buttons_3.get_value())) + "\n";
    text_to_send = text_to_send + at_pos;
    num_left = num_left - 1;
    entry_atom_num->set_text(to_string(num_left));
    if(num_left == 0)
    {
        final_txt = info_message(text_to_send+"\n\nwill be added to script.");
        text_to_send = "";
        hide();
    }
}

Glib::ustring add_atomic_pos_window:: info_message(Glib::ustring message)
{
    Gtk::MessageDialog dialog(message, false, Gtk::MESSAGE_INFO, Gtk::BUTTONS_CLOSE, true);
    dialog.set_transient_for(*this);
    dialog.run();
    return(text_to_send);
}
