/*
 * Copyright 2019 Oliwia Chrapowicka
 *
 * This file is part of QESG.
 *
 * QESG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * QESG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with QESG.  If not, see <http://www.gnu.org/licenses/>.
 */


#include"window_main.h"

using namespace std;


window_main::window_main(int width = 1100, int height = 800, Glib::ustring window_name = "QUANTUM ESPRESSO SCRIPT GENERATOR")
{
	set_default_size(width,height);
	set_title(window_name);
	signal_delete_event().connect(sigc::mem_fun(*this, &window_main::delete_event));

	//MAIN BOX
	Gtk::Box *mbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0));
	add(*mbox);

	//MENUBAR
	Gtk::MenuBar *menubar = Gtk::manage(new Gtk::MenuBar());
	mbox->pack_start(*menubar, Gtk::PACK_SHRINK, 0);

	//MENUBAR -> File
	Gtk::MenuItem *menuitem_file = Gtk::manage(new Gtk::MenuItem("_File",true));
	menubar->append(*menuitem_file);
	Gtk::Menu *submenu_file = Gtk::manage(new Gtk::Menu());
	menuitem_file->set_submenu(*submenu_file);
	//MENUBAR -> File -> Open script
	Gtk::MenuItem *menuitem_open_script = Gtk::manage(new Gtk::MenuItem("Open script...",true)); //Open existing script to edit
	menuitem_open_script->signal_activate().connect(sigc::mem_fun(*this, &window_main::click_on_open_script)); //
	submenu_file->append(*menuitem_open_script);
	//MENUBAR -> File -> Set basic script
	Gtk::MenuItem *menuitem_set_basic_script = Gtk::manage(new Gtk::MenuItem("Set basic script",true)); //Choose a file to be set as basic script
	menuitem_set_basic_script->signal_activate().connect(sigc::mem_fun(*this, &window_main::click_on_set_basic_script)); //
	submenu_file->append(*menuitem_set_basic_script);
	//MENUBAR -> File -> Quit
	Gtk::MenuItem *menuitem_quit = Gtk::manage(new Gtk::MenuItem("_Quit",true)); //Quit option
	menuitem_quit->signal_activate().connect(sigc::mem_fun(*this, &window_main::click_on_quit)); //
	submenu_file->append(*menuitem_quit);

	//MENUBAR -> Edit
	Gtk::MenuItem *menuitem_edit = Gtk::manage(new Gtk::MenuItem("_Edit",true));
	menubar->append(*menuitem_edit);
	Gtk::Menu *submenu_edit = Gtk::manage(new Gtk::Menu());
	menuitem_edit->set_submenu(*submenu_edit);
	//MENUBAR -> Edit -> Choose default folder
	Gtk::MenuItem *menuitem_choose_folder = Gtk::manage(new Gtk::MenuItem("Choose default folder...",true)); //Choose default folder to save results in
	menuitem_choose_folder->signal_activate().connect(sigc::mem_fun(*this, &window_main::click_on_choose_folder)); //
	submenu_edit->append(*menuitem_choose_folder);

	//MENUBAR -> Help
	Gtk::MenuItem *menuitem_help = Gtk::manage(new Gtk::MenuItem("_Help",true));
	menubar->append(*menuitem_help);
	Gtk::Menu *submenu_help = Gtk::manage(new Gtk::Menu());
	menuitem_help->set_submenu(*submenu_help);

	Gtk::MenuItem *menuitem_subhelp1 = Gtk::manage(new Gtk::MenuItem("Link for 'TABLE OF CONTENTS'",true));
	submenu_help->append(*menuitem_subhelp1);
	menuitem_subhelp1->signal_activate().connect(sigc::mem_fun(*this, &window_main::click_on_help1)); //

	Gtk::MenuItem *menuitem_subhelp2 = Gtk::manage(new Gtk::MenuItem("Link for tutorial",true));
	submenu_help->append(*menuitem_subhelp2);
	menuitem_subhelp2->signal_activate().connect(sigc::mem_fun(*this, &window_main::click_on_help2)); //


	//CONTROL TAB:
		//GRID
		Gtk::Grid *grid_control = Gtk::manage(new Gtk::Grid());
		grid_control->set_border_width(20);

			//calculation - tree model columns
			calculation_list = Gtk::ListStore::create(calculation_columns);
			calculation_combobox.set_model(calculation_list);
			Gtk::TreeModel::Row calculation_rows = *(calculation_list->append());
	  		calculation_rows[calculation_columns.columns_id] = "nscf";
			calculation_rows = *(calculation_list->append());
	  		calculation_rows[calculation_columns.columns_id] = "scf";
			calculation_rows = *(calculation_list->append());
	  		calculation_rows[calculation_columns.columns_id] = "bands";
	  		calculation_rows = *(calculation_list->append());
			calculation_rows[calculation_columns.columns_id] = "relax";
			calculation_rows = *(calculation_list->append());
			calculation_rows[calculation_columns.columns_id] = "md";
			calculation_rows = *(calculation_list->append());
			calculation_rows[calculation_columns.columns_id] = "vc-relax";
			calculation_rows = *(calculation_list->append());
			calculation_rows[calculation_columns.columns_id] = "vc-md";

			calculation_combobox.pack_start(calculation_columns.columns_id);
			calculation_combobox.set_entry_text_column(calculation_columns.columns_id);
			calculation_combobox.set_active(1);

			calculation_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &calculation_value, calculation_columns.columns_id, &calculation_combobox, calculation_id, 0, "&CONTROL\n" ));
			Gtk::Label *label_calculation = Gtk::manage(new Gtk::Label("calculation = "));
			grid_control->attach(*label_calculation,0,0,5,5);
			grid_control->attach_next_to(calculation_combobox,*label_calculation,Gtk::POS_RIGHT,5,5);

			//verbosity - tree model columns
			verbosity_list = Gtk::ListStore::create(verbosity_columns);
			verbosity_combobox.set_model(verbosity_list);
			Gtk::TreeModel::Row verbosity_rows = *(verbosity_list->append());
	  		verbosity_rows[verbosity_columns.columns_id] = "high";
			verbosity_rows = *(verbosity_list->append());
	  		verbosity_rows[verbosity_columns.columns_id] = "low";

			verbosity_combobox.pack_start(verbosity_columns.columns_id);
			verbosity_combobox.set_entry_text_column(verbosity_columns.columns_id);
		  	verbosity_combobox.set_active(1);

			verbosity_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &verbosity_value, verbosity_columns.columns_id, &verbosity_combobox, verbosity_id, 0, "&CONTROL\n" ));
			Gtk::Label *label_verbosity = Gtk::manage(new Gtk::Label("verbosity = "));
			grid_control->attach_next_to(*label_verbosity, *label_calculation,Gtk::POS_BOTTOM,5,5);
			grid_control->attach_next_to(verbosity_combobox,*label_verbosity,Gtk::POS_RIGHT,5,5);

			//restart_mode - tree model columns
			restart_mode_list = Gtk::ListStore::create(restart_mode_columns);
			restart_mode_combobox.set_model(restart_mode_list);
			Gtk::TreeModel::Row restart_mode_rows = *(restart_mode_list->append());
	  		restart_mode_rows[restart_mode_columns.columns_id] = "restart";
			restart_mode_rows = *(restart_mode_list->append());
	  		restart_mode_rows[restart_mode_columns.columns_id] = "from_scratch";

			restart_mode_combobox.pack_start(restart_mode_columns.columns_id);
			restart_mode_combobox.set_entry_text_column(restart_mode_columns.columns_id);
		  	restart_mode_combobox.set_active(1);

			restart_mode_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &restart_mode_value, restart_mode_columns.columns_id, &restart_mode_combobox, restart_mode_id, 0, "&CONTROL\n" ));
			Gtk::Label *label_restart_mode = Gtk::manage(new Gtk::Label("restart_mode = "));
			grid_control->attach_next_to(*label_restart_mode, *label_verbosity,Gtk::POS_BOTTOM,5,5);
			grid_control->attach_next_to(restart_mode_combobox,*label_restart_mode,Gtk::POS_RIGHT,5,5);

			//disk_io - tree model columns
			disk_io_list = Gtk::ListStore::create(disk_io_columns);
			disk_io_combobox.set_model(disk_io_list);
			Gtk::TreeModel::Row disk_io_rows = *(disk_io_list->append());
			disk_io_rows[disk_io_columns.columns_id] = "none";
			disk_io_rows = *(disk_io_list->append());
			disk_io_rows[disk_io_columns.columns_id] = "low";
			disk_io_rows = *(disk_io_list->append());
			disk_io_rows[disk_io_columns.columns_id] = "medium";
			disk_io_rows = *(disk_io_list->append());
			disk_io_rows[disk_io_columns.columns_id] = "high";

			disk_io_combobox.pack_start(disk_io_columns.columns_id);
			disk_io_combobox.set_entry_text_column(disk_io_columns.columns_id);
			disk_io_combobox.set_active(1);

			disk_io_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &disk_io_value, disk_io_columns.columns_id, &disk_io_combobox, disk_io_id, 0, "&CONTROL\n" ));
			Gtk::Label *label_disk_io = Gtk::manage(new Gtk::Label("disk_io = "));
			grid_control->attach_next_to(*label_disk_io, *label_restart_mode,Gtk::POS_BOTTOM,5,5);
			grid_control->attach_next_to(disk_io_combobox,*label_disk_io,Gtk::POS_RIGHT,5,5);

			help_link1.set_uri("http://web.mit.edu/espresso_v6.1/i386_linux26/qe-6.1/Doc/INPUT_PW.html");
			help_link1.set_label("");

			help_link2.set_uri("https://www.quantum-espresso.org/resources/tutorials");
			help_link2.set_label("");



	//SYSTEM TAB:
		//GRID
		Gtk::Grid *grid_system = Gtk::manage(new Gtk::Grid());
		grid_system->set_border_width(20);

			//nat
			sbutton_nat.set_adjustment(adj_sb_nat);
			sbutton_nat.set_snap_to_ticks();
			sbutton_nat.set_numeric();
			sbutton_nat.set_digits(0);
			grid_system->attach(*label_nat,0,0,5,5);
			grid_system->attach_next_to(sbutton_nat,*label_nat,Gtk::POS_RIGHT,5,5);
			sbutton_nat.signal_value_changed().connect(sigc::mem_fun(*this, &window_main::changed_nat) );

			//ntyp
			sbutton_ntyp.set_adjustment(adj_sb_ntyp);
			sbutton_ntyp.set_snap_to_ticks();
			sbutton_ntyp.set_numeric();
			sbutton_ntyp.set_digits(0);
			grid_system->attach_next_to(*label_ntyp, *label_nat,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(sbutton_ntyp,*label_ntyp,Gtk::POS_RIGHT,5,5);
			sbutton_ntyp.signal_value_changed().connect(sigc::mem_fun(*this, &window_main::changed_ntyp) );


			//ibrav - tree model columns
			ibrav_list = Gtk::ListStore::create(ibrav_columns);
			ibrav_combobox.set_model(ibrav_list);
			Gtk::TreeModel::Row ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "1";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "0";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "2";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "3";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "4";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "5";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "-5";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "6";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "7";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "8";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "9";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "-9";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "10";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "11";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "12";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "-12";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "13";
			ibrav_rows = *(ibrav_list->append());
			ibrav_rows[ibrav_columns.columns_id] = "14";

			ibrav_combobox.pack_start(ibrav_columns.columns_id);
			ibrav_combobox.set_entry_text_column(ibrav_columns.columns_id);
			ibrav_combobox.set_active(1);

			ibrav_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &ibrav_value, ibrav_columns.columns_id, &ibrav_combobox, ibrav_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_ibrav = Gtk::manage(new Gtk::Label("ibrav = "));
			grid_system->attach_next_to(*label_ibrav, *label_ntyp,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(ibrav_combobox,*label_ibrav,Gtk::POS_RIGHT,5,5);


			//occupations - tree model columns
			occupations_list = Gtk::ListStore::create(occupations_columns);
			occupations_combobox.set_model(occupations_list);
			Gtk::TreeModel::Row occupations_rows = *(occupations_list->append());
			occupations_rows[occupations_columns.columns_id] = "tetrahedra";
			occupations_rows = *(occupations_list->append());
			occupations_rows[occupations_columns.columns_id] = "smearing";
			occupations_rows = *(occupations_list->append());
			occupations_rows[occupations_columns.columns_id] = "tetrahedra_lin";
			occupations_rows = *(occupations_list->append());
			occupations_rows[occupations_columns.columns_id] = "tetrahedra_opt";
			occupations_rows = *(occupations_list->append());
			occupations_rows[occupations_columns.columns_id] = "fixed";
			occupations_rows = *(occupations_list->append());
			occupations_rows[occupations_columns.columns_id] = "from_input";

			occupations_combobox.pack_start(occupations_columns.columns_id);
			occupations_combobox.set_entry_text_column(occupations_columns.columns_id);
			occupations_combobox.set_active(1);

			occupations_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &occupations_value, occupations_columns.columns_id, &occupations_combobox, occupations_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_occupations = Gtk::manage(new Gtk::Label("occupations = "));
			grid_system->attach_next_to(*label_occupations, *label_ibrav, Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(occupations_combobox,*label_occupations, Gtk::POS_RIGHT,5,5);

			//smearing - tree model columns
			smearing_list = Gtk::ListStore::create(smearing_columns);
			smearing_combobox.set_model(smearing_list);
			Gtk::TreeModel::Row smearing_rows = *(smearing_list->append());
			smearing_rows[smearing_columns.columns_id] = "methfessel-paxton";
			smearing_rows = *(smearing_list->append());
			smearing_rows[smearing_columns.columns_id] = "gaussian";
			smearing_rows = *(smearing_list->append());
			smearing_rows[smearing_columns.columns_id] = "marzari-vanderbilt";
			smearing_rows = *(smearing_list->append());
			smearing_rows[smearing_columns.columns_id] = "fermi-dirac";

			smearing_combobox.pack_start(smearing_columns.columns_id);
			smearing_combobox.set_entry_text_column(smearing_columns.columns_id);
			smearing_combobox.set_active(1);

			smearing_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &smearing_value, smearing_columns.columns_id, &smearing_combobox, smearing_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_smearing = Gtk::manage(new Gtk::Label("smearing = "));
			grid_system->attach_next_to(*label_smearing, *label_occupations,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(smearing_combobox,*label_smearing,Gtk::POS_RIGHT,5,5);

			//exxdiv_treatment - tree model columns
			exxdiv_treatment_list = Gtk::ListStore::create(exxdiv_treatment_columns);
			exxdiv_treatment_combobox.set_model(exxdiv_treatment_list);
			Gtk::TreeModel::Row exxdiv_treatment_rows = *(exxdiv_treatment_list->append());
			exxdiv_treatment_rows[exxdiv_treatment_columns.columns_id] = "vcut_spherical";
			exxdiv_treatment_rows = *(exxdiv_treatment_list->append());
			exxdiv_treatment_rows[exxdiv_treatment_columns.columns_id] = "gygi-baldereschi";
			exxdiv_treatment_rows = *(exxdiv_treatment_list->append());
			exxdiv_treatment_rows[exxdiv_treatment_columns.columns_id] = "vcut_ws";
			exxdiv_treatment_rows = *(exxdiv_treatment_list->append());
			exxdiv_treatment_rows[exxdiv_treatment_columns.columns_id] = "none";

			exxdiv_treatment_combobox.pack_start(exxdiv_treatment_columns.columns_id);
			exxdiv_treatment_combobox.set_entry_text_column(exxdiv_treatment_columns.columns_id);
			exxdiv_treatment_combobox.set_active(1);

			exxdiv_treatment_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &exxdiv_treatment_value, exxdiv_treatment_columns.columns_id, &exxdiv_treatment_combobox, exxdiv_treatment_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_exxdiv_treatment = Gtk::manage(new Gtk::Label("exxdiv_treatment = "));
			grid_system->attach_next_to(*label_exxdiv_treatment, *label_smearing,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(exxdiv_treatment_combobox,*label_exxdiv_treatment,Gtk::POS_RIGHT,5,5);

			//U_projection_type - tree model columns
			U_projection_type_list = Gtk::ListStore::create(U_projection_type_columns);
			U_projection_type_combobox.set_model(U_projection_type_list);
			Gtk::TreeModel::Row U_projection_type_rows = *(U_projection_type_list->append());
			U_projection_type_rows[U_projection_type_columns.columns_id] = "ortho-atomic";
			U_projection_type_rows = *(U_projection_type_list->append());
			U_projection_type_rows[U_projection_type_columns.columns_id] = "atomic";
			U_projection_type_rows = *(U_projection_type_list->append());
			U_projection_type_rows[U_projection_type_columns.columns_id] = "norm-atomic";
			U_projection_type_rows = *(U_projection_type_list->append());
			U_projection_type_rows[U_projection_type_columns.columns_id] = "file";
			U_projection_type_rows = *(U_projection_type_list->append());
			U_projection_type_rows[U_projection_type_columns.columns_id] = "psuedo";

			U_projection_type_combobox.pack_start(U_projection_type_columns.columns_id);
			U_projection_type_combobox.set_entry_text_column(U_projection_type_columns.columns_id);
			U_projection_type_combobox.set_active(1);

			U_projection_type_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &U_projection_type_value, U_projection_type_columns.columns_id, &U_projection_type_combobox, U_projection_type_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_U_projection_type = Gtk::manage(new Gtk::Label("U_projection_type = "));
			grid_system->attach_next_to(*label_U_projection_type, *label_exxdiv_treatment,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(U_projection_type_combobox,*label_U_projection_type,Gtk::POS_RIGHT,5,5);

			//constrained_magnetization - tree model columns
			constrained_magnetization_list = Gtk::ListStore::create(constrained_magnetization_columns);
			constrained_magnetization_combobox.set_model(constrained_magnetization_list);
			Gtk::TreeModel::Row constrained_magnetization_rows = *(constrained_magnetization_list->append());
			constrained_magnetization_rows[constrained_magnetization_columns.columns_id] = "atomic";
			constrained_magnetization_rows = *(constrained_magnetization_list->append());
			constrained_magnetization_rows[constrained_magnetization_columns.columns_id] = "none";
			constrained_magnetization_rows = *(constrained_magnetization_list->append());
			constrained_magnetization_rows[constrained_magnetization_columns.columns_id] = "total";
			constrained_magnetization_rows = *(constrained_magnetization_list->append());
			constrained_magnetization_rows[constrained_magnetization_columns.columns_id] = "total direction";
			constrained_magnetization_rows = *(constrained_magnetization_list->append());
			constrained_magnetization_rows[constrained_magnetization_columns.columns_id] = "atomic direction";

			constrained_magnetization_combobox.pack_start(constrained_magnetization_columns.columns_id);
			constrained_magnetization_combobox.set_entry_text_column(constrained_magnetization_columns.columns_id);
			constrained_magnetization_combobox.set_active(1);

			constrained_magnetization_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &constrained_magnetization_value, constrained_magnetization_columns.columns_id, &constrained_magnetization_combobox, constrained_magnetization_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_constrained_magnetization = Gtk::manage(new Gtk::Label("constrained_magnetization = "));
			grid_system->attach_next_to(*label_constrained_magnetization, *label_U_projection_type,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(constrained_magnetization_combobox,*label_constrained_magnetization,Gtk::POS_RIGHT,5,5);
			//assume_isolated - tree model columns
			assume_isolated_list = Gtk::ListStore::create(assume_isolated_columns);
			assume_isolated_combobox.set_model(assume_isolated_list);
			Gtk::TreeModel::Row assume_isolated_rows = *(assume_isolated_list->append());
			assume_isolated_rows[assume_isolated_columns.columns_id] = "makov-payne";
			assume_isolated_rows = *(assume_isolated_list->append());
			assume_isolated_rows[assume_isolated_columns.columns_id] = "none";
			assume_isolated_rows = *(assume_isolated_list->append());
			assume_isolated_rows[assume_isolated_columns.columns_id] = "martyna-tuckerman";
			assume_isolated_rows = *(assume_isolated_list->append());
			assume_isolated_rows[assume_isolated_columns.columns_id] = "esm";

			assume_isolated_combobox.pack_start(assume_isolated_columns.columns_id);
			assume_isolated_combobox.set_entry_text_column(assume_isolated_columns.columns_id);
			assume_isolated_combobox.set_active(1);

			assume_isolated_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &assume_isolated_value, assume_isolated_columns.columns_id, &assume_isolated_combobox, assume_isolated_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_assume_isolated = Gtk::manage(new Gtk::Label("assume_isolated = "));
			grid_system->attach_next_to(*label_assume_isolated, *label_constrained_magnetization,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(assume_isolated_combobox,*label_assume_isolated,Gtk::POS_RIGHT,5,5);

			//esm_bc - tree model columns
			esm_bc_list = Gtk::ListStore::create(esm_bc_columns);
			esm_bc_combobox.set_model(esm_bc_list);
			Gtk::TreeModel::Row esm_bc_rows = *(esm_bc_list->append());
			esm_bc_rows[esm_bc_columns.columns_id] = "bc1";
			esm_bc_rows = *(esm_bc_list->append());
			esm_bc_rows[esm_bc_columns.columns_id] = "pbc";
			esm_bc_rows = *(esm_bc_list->append());
			esm_bc_rows[esm_bc_columns.columns_id] = "bc2";
			esm_bc_rows = *(esm_bc_list->append());
			esm_bc_rows[esm_bc_columns.columns_id] = "bc3";

			esm_bc_combobox.pack_start(esm_bc_columns.columns_id);
			esm_bc_combobox.set_entry_text_column(esm_bc_columns.columns_id);
			esm_bc_combobox.set_active(1);

			esm_bc_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &esm_bc_value, esm_bc_columns.columns_id, &esm_bc_combobox, esm_bc_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_esm_bc = Gtk::manage(new Gtk::Label("esm_bc = "));
			grid_system->attach_next_to(*label_esm_bc, *label_assume_isolated,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(esm_bc_combobox,*label_esm_bc,Gtk::POS_RIGHT,5,5);

			//vdw_corr - tree model columns
			vdw_corr_list = Gtk::ListStore::create(vdw_corr_columns);
			vdw_corr_combobox.set_model(vdw_corr_list);
			Gtk::TreeModel::Row vdw_corr_rows = *(vdw_corr_list->append());
			vdw_corr_rows[vdw_corr_columns.columns_id] = "grimme-d2";
			vdw_corr_rows = *(vdw_corr_list->append());
			vdw_corr_rows[vdw_corr_columns.columns_id] = "none";
			vdw_corr_rows = *(vdw_corr_list->append());
			vdw_corr_rows[vdw_corr_columns.columns_id] = "TS";
			vdw_corr_rows = *(vdw_corr_list->append());
			vdw_corr_rows[vdw_corr_columns.columns_id] = "XDM";

			vdw_corr_combobox.pack_start(vdw_corr_columns.columns_id);
			vdw_corr_combobox.set_entry_text_column(vdw_corr_columns.columns_id);
			vdw_corr_combobox.set_active(1);

			vdw_corr_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &vdw_corr_value, vdw_corr_columns.columns_id, &vdw_corr_combobox, vdw_corr_id, 0, "&SYSTEM\n" ));
			Gtk::Label *label_vdw_corr = Gtk::manage(new Gtk::Label("vdw_corr = "));
			grid_system->attach_next_to(*label_vdw_corr, *label_esm_bc,Gtk::POS_BOTTOM,5,5);
			grid_system->attach_next_to(vdw_corr_combobox,*label_vdw_corr,Gtk::POS_RIGHT,5,5);


	//ELECTRONS TAB:
		//GRID
		Gtk::Grid *grid_electrons = Gtk::manage(new Gtk::Grid());
		grid_electrons->set_border_width(20);

			//mixing_mode - tree model columns
			mixing_mode_list = Gtk::ListStore::create(mixing_mode_columns);
			mixing_mode_combobox.set_model(mixing_mode_list);
			Gtk::TreeModel::Row mixing_mode_rows = *(mixing_mode_list->append());
			mixing_mode_rows[mixing_mode_columns.columns_id] = "TF";
			mixing_mode_rows = *(mixing_mode_list->append());
			mixing_mode_rows[mixing_mode_columns.columns_id] = "plain";
			mixing_mode_rows = *(mixing_mode_list->append());
			mixing_mode_rows[mixing_mode_columns.columns_id] = "local-TF";

			mixing_mode_combobox.pack_start(mixing_mode_columns.columns_id);
			mixing_mode_combobox.set_entry_text_column(mixing_mode_columns.columns_id);
			mixing_mode_combobox.set_active(1);

			mixing_mode_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &mixing_mode_value, mixing_mode_columns.columns_id, &mixing_mode_combobox, mixing_mode_id, 0, "&ELECTRONS\n" ));
			Gtk::Label *label_mixing_mode = Gtk::manage(new Gtk::Label("mixing_mode = "));
			grid_electrons->attach(*label_mixing_mode,0,0,5,5);
			grid_electrons->attach_next_to(mixing_mode_combobox,*label_mixing_mode,Gtk::POS_RIGHT,5,5);

			//diagonalization - tree model columns
			diagonalization_list = Gtk::ListStore::create(diagonalization_columns);
			diagonalization_combobox.set_model(diagonalization_list);
			Gtk::TreeModel::Row diagonalization_rows = *(diagonalization_list->append());
			diagonalization_rows[diagonalization_columns.columns_id] = "cg";
			diagonalization_rows = *(diagonalization_list->append());
			diagonalization_rows[diagonalization_columns.columns_id] = "david";
			diagonalization_rows = *(diagonalization_list->append());
			diagonalization_rows[diagonalization_columns.columns_id] = "cg-serial";

			diagonalization_combobox.pack_start(diagonalization_columns.columns_id);
			diagonalization_combobox.set_entry_text_column(diagonalization_columns.columns_id);
			diagonalization_combobox.set_active(1);

			diagonalization_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &diagonalization_value, diagonalization_columns.columns_id, &diagonalization_combobox, diagonalization_id, 0, "&ELECTRONS\n" ));
			Gtk::Label *label_diagonalization = Gtk::manage(new Gtk::Label("diagonalization = "));
			grid_electrons->attach_next_to(*label_diagonalization, *label_mixing_mode,Gtk::POS_BOTTOM,5,5);
			grid_electrons->attach_next_to(diagonalization_combobox,*label_diagonalization, Gtk::POS_RIGHT,5,5);

			//efield_phase - tree model columns
			efield_phase_list = Gtk::ListStore::create(efield_phase_columns);
			efield_phase_combobox.set_model(efield_phase_list);
			Gtk::TreeModel::Row efield_phase_rows = *(efield_phase_list->append());
			efield_phase_rows[efield_phase_columns.columns_id] = "read";
			efield_phase_rows = *(efield_phase_list->append());
			efield_phase_rows[efield_phase_columns.columns_id] = "none";
			efield_phase_rows = *(efield_phase_list->append());
			efield_phase_rows[efield_phase_columns.columns_id] = "write";

			efield_phase_combobox.pack_start(efield_phase_columns.columns_id);
			efield_phase_combobox.set_entry_text_column(efield_phase_columns.columns_id);
			efield_phase_combobox.set_active(1);

			efield_phase_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &efield_phase_value, efield_phase_columns.columns_id, &efield_phase_combobox, efield_phase_id, 0, "&ELECTRONS\n" ));
			Gtk::Label *label_efield_phase = Gtk::manage(new Gtk::Label("efield_phase = "));
			grid_electrons->attach_next_to(*label_efield_phase, *label_diagonalization,Gtk::POS_BOTTOM,5,5);
			grid_electrons->attach_next_to(efield_phase_combobox,*label_efield_phase, Gtk::POS_RIGHT,5,5);

			//startingpot - tree model columns
			startingpot_list = Gtk::ListStore::create(startingpot_columns);
			startingpot_combobox.set_model(startingpot_list);
			Gtk::TreeModel::Row startingpot_rows = *(startingpot_list->append());
			startingpot_rows[startingpot_columns.columns_id] = "file";
			startingpot_rows = *(startingpot_list->append());
			startingpot_rows[startingpot_columns.columns_id] = "atomic";

			startingpot_combobox.pack_start(startingpot_columns.columns_id);
			startingpot_combobox.set_entry_text_column(startingpot_columns.columns_id);
			startingpot_combobox.set_active(1);

			startingpot_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &startingpot_value, startingpot_columns.columns_id, &startingpot_combobox, startingpot_id, 0, "&ELECTRONS\n" ));
			Gtk::Label *label_startingpot = Gtk::manage(new Gtk::Label("startingpot = "));
			grid_electrons->attach_next_to(*label_startingpot, *label_efield_phase,Gtk::POS_BOTTOM,5,5);
			grid_electrons->attach_next_to(startingpot_combobox,*label_startingpot, Gtk::POS_RIGHT,5,5);

			//startingwfc - tree model columns
			startingwfc_list = Gtk::ListStore::create(startingwfc_columns);
			startingwfc_combobox.set_model(startingwfc_list);
			Gtk::TreeModel::Row startingwfc_rows = *(startingwfc_list->append());
			startingwfc_rows[startingwfc_columns.columns_id] = "file";
			startingwfc_rows = *(startingwfc_list->append());
			startingwfc_rows[startingwfc_columns.columns_id] = "atomic+random";
			startingwfc_rows = *(startingwfc_list->append());
			startingwfc_rows[startingwfc_columns.columns_id] = "atomic";
			startingwfc_rows = *(startingwfc_list->append());
			startingwfc_rows[startingwfc_columns.columns_id] = "random";

			startingwfc_combobox.pack_start(startingwfc_columns.columns_id);
			startingwfc_combobox.set_entry_text_column(startingwfc_columns.columns_id);
			startingwfc_combobox.set_active(1);

			startingwfc_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &startingwfc_value, startingwfc_columns.columns_id, &startingwfc_combobox, startingwfc_id, 0, "&ELECTRONS\n" ));
			Gtk::Label *label_startingwfc = Gtk::manage(new Gtk::Label("startingwfc = "));
			grid_electrons->attach_next_to(*label_startingwfc, *label_startingpot,Gtk::POS_BOTTOM,5,5);
			grid_electrons->attach_next_to(startingwfc_combobox,*label_startingwfc, Gtk::POS_RIGHT,5,5);


	//IONS TAB:
		//GRID
		Gtk::Grid *grid_ions = Gtk::manage(new Gtk::Grid());
		grid_ions->set_border_width(20);

			//ion_positions - tree model columns
			ion_positions_list = Gtk::ListStore::create(ion_positions_columns);
			ion_positions_combobox.set_model(ion_positions_list);
			Gtk::TreeModel::Row ion_positions_rows = *(ion_positions_list->append());
			ion_positions_rows[ion_positions_columns.columns_id] = "from_input";
			ion_positions_rows = *(ion_positions_list->append());
			ion_positions_rows[ion_positions_columns.columns_id] = "default";

			ion_positions_combobox.pack_start(ion_positions_columns.columns_id);
			ion_positions_combobox.set_entry_text_column(ion_positions_columns.columns_id);
			ion_positions_combobox.set_active(1);

			ion_positions_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &ion_positions_value, ion_positions_columns.columns_id, &ion_positions_combobox, ion_positions_id, 0, "&IONS\n" ));
			Gtk::Label *label_ion_positions = Gtk::manage(new Gtk::Label("ion_positions = "));
			grid_ions->attach(*label_ion_positions,0,0,5,5);
			grid_ions->attach_next_to(ion_positions_combobox,*label_ion_positions,Gtk::POS_RIGHT,5,5);

			//pot_extrapolation - tree model columns
			pot_extrapolation_list = Gtk::ListStore::create(pot_extrapolation_columns);
			pot_extrapolation_combobox.set_model(pot_extrapolation_list);
			Gtk::TreeModel::Row pot_extrapolation_rows = *(pot_extrapolation_list->append());
			pot_extrapolation_rows[pot_extrapolation_columns.columns_id] = "none";
			pot_extrapolation_rows = *(pot_extrapolation_list->append());
			pot_extrapolation_rows[pot_extrapolation_columns.columns_id] = "atomic";
			pot_extrapolation_rows = *(pot_extrapolation_list->append());
			pot_extrapolation_rows[pot_extrapolation_columns.columns_id] = "first_order";
			pot_extrapolation_rows = *(pot_extrapolation_list->append());
			pot_extrapolation_rows[pot_extrapolation_columns.columns_id] = "second_order";

			pot_extrapolation_combobox.pack_start(pot_extrapolation_columns.columns_id);
			pot_extrapolation_combobox.set_entry_text_column(pot_extrapolation_columns.columns_id);
			pot_extrapolation_combobox.set_active(1);

			pot_extrapolation_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &pot_extrapolation_value, pot_extrapolation_columns.columns_id, &pot_extrapolation_combobox, pot_extrapolation_id, 0, "&IONS\n" ));
			Gtk::Label *label_pot_extrapolation = Gtk::manage(new Gtk::Label("pot_extrapolation = "));
			grid_ions->attach_next_to(*label_pot_extrapolation, *label_ion_positions, Gtk::POS_BOTTOM,5,5);
			grid_ions->attach_next_to(pot_extrapolation_combobox,*label_pot_extrapolation,Gtk::POS_RIGHT,5,5);

			//wfc_extrapolation - tree model columns
			wfc_extrapolation_list = Gtk::ListStore::create(wfc_extrapolation_columns);
			wfc_extrapolation_combobox.set_model(wfc_extrapolation_list);
			Gtk::TreeModel::Row wfc_extrapolation_rows = *(wfc_extrapolation_list->append());
			wfc_extrapolation_rows[wfc_extrapolation_columns.columns_id] = "first_order";
			wfc_extrapolation_rows = *(wfc_extrapolation_list->append());
			wfc_extrapolation_rows[wfc_extrapolation_columns.columns_id] = "none";
			wfc_extrapolation_rows = *(wfc_extrapolation_list->append());
			wfc_extrapolation_rows[wfc_extrapolation_columns.columns_id] = "second_order";

			wfc_extrapolation_combobox.pack_start(wfc_extrapolation_columns.columns_id);
			wfc_extrapolation_combobox.set_entry_text_column(wfc_extrapolation_columns.columns_id);
			wfc_extrapolation_combobox.set_active(1);

			wfc_extrapolation_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &wfc_extrapolation_value, wfc_extrapolation_columns.columns_id, &wfc_extrapolation_combobox, wfc_extrapolation_id, 0, "&IONS\n" ));
			Gtk::Label *label_wfc_extrapolation = Gtk::manage(new Gtk::Label("wfc_extrapolation = "));
			grid_ions->attach_next_to(*label_wfc_extrapolation, *label_pot_extrapolation, Gtk::POS_BOTTOM,5,5);
			grid_ions->attach_next_to(wfc_extrapolation_combobox,*label_wfc_extrapolation,Gtk::POS_RIGHT,5,5);

			//ion_temperature - tree model columns
			ion_temperature_list = Gtk::ListStore::create(ion_temperature_columns);
			ion_temperature_combobox.set_model(ion_temperature_list);
			Gtk::TreeModel::Row ion_temperature_rows = *(ion_temperature_list->append());
			ion_temperature_rows[ion_temperature_columns.columns_id] = "rescaling";
			ion_temperature_rows = *(ion_temperature_list->append());
			ion_temperature_rows[ion_temperature_columns.columns_id] = "not_controlled";
			ion_temperature_rows = *(ion_temperature_list->append());
			ion_temperature_rows[ion_temperature_columns.columns_id] = "rescale-v";
			ion_temperature_rows = *(ion_temperature_list->append());
			ion_temperature_rows[ion_temperature_columns.columns_id] = "rescale-T";
			ion_temperature_rows = *(ion_temperature_list->append());
			ion_temperature_rows[ion_temperature_columns.columns_id] = "reduce-T";
			ion_temperature_rows = *(ion_temperature_list->append());
			ion_temperature_rows[ion_temperature_columns.columns_id] = "berendsen";
			ion_temperature_rows = *(ion_temperature_list->append());
			ion_temperature_rows[ion_temperature_columns.columns_id] = "andersen";
			ion_temperature_rows = *(ion_temperature_list->append());
			ion_temperature_rows[ion_temperature_columns.columns_id] = "initial";

			ion_temperature_combobox.pack_start(ion_temperature_columns.columns_id);
			ion_temperature_combobox.set_entry_text_column(ion_temperature_columns.columns_id);
			ion_temperature_combobox.set_active(1);

			ion_temperature_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &ion_temperature_value, ion_temperature_columns.columns_id, &ion_temperature_combobox, ion_temperature_id, 0, "&IONS\n" ));
			Gtk::Label *label_ion_temperature = Gtk::manage(new Gtk::Label("ion_temperature = "));
			grid_ions->attach_next_to(*label_ion_temperature, *label_wfc_extrapolation, Gtk::POS_BOTTOM,5,5);
			grid_ions->attach_next_to(ion_temperature_combobox,*label_ion_temperature,Gtk::POS_RIGHT,5,5);


	//CELL TAB:
		//GRID
		Gtk::Grid *grid_cell = Gtk::manage(new Gtk::Grid());
		grid_cell->set_border_width(20);

			//cell_dofree - tree model columns
			cell_dofree_list = Gtk::ListStore::create(cell_dofree_columns);
			cell_dofree_combobox.set_model(cell_dofree_list);
			Gtk::TreeModel::Row cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "x";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "all";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "y";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "z";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "xy";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "xz";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "yz";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "xyz";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "shape";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "volume";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "2Dxy";
			cell_dofree_rows = *(cell_dofree_list->append());
			cell_dofree_rows[cell_dofree_columns.columns_id] = "2Dshape";

			cell_dofree_combobox.pack_start(cell_dofree_columns.columns_id);
			cell_dofree_combobox.set_entry_text_column(cell_dofree_columns.columns_id);
			cell_dofree_combobox.set_active(1);

			cell_dofree_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &cell_dofree_value, cell_dofree_columns.columns_id, &cell_dofree_combobox, cell_dofree_id, 0, "&CELL\n" ));
			Gtk::Label *label_cell_dofree = Gtk::manage(new Gtk::Label("cell_dofree = "));
			grid_cell->attach(*label_cell_dofree,0,0,5,5);
			grid_cell->attach_next_to(cell_dofree_combobox,*label_cell_dofree,Gtk::POS_RIGHT,5,5);

	//CELL_PARAMETERES TAB:
		//BOXES
		box_cell_parameters.set_orientation(Gtk::ORIENTATION_VERTICAL);
		box_cell_parameters.set_border_width(20);
		box_cell_parameters.set_spacing(10);
		box_cell_parameters.pack_start(frame_vectors,false,false,1);
		box_cell_parameters.pack_start(box_parameters,false,false,1);
		box_cell_parameters.pack_end(help_link1,false,false,1);
		box_cell_parameters.pack_end(help_link2,false,false,1);

		frame_vectors.add(box_vectors);
		frame_vectors.set_label("Crystal lattice vectors (in cartesian axis):");

		box_vectors.set_border_width(10);
		box_vectors.set_spacing(30);
		box_vectors.set_orientation(Gtk::ORIENTATION_VERTICAL);
		box_vectors.pack_start(box_v1,false,false,1);
		box_vectors.pack_start(box_v2,false,false,1);
		box_vectors.pack_start(box_v3,false,false,1);
		box_vectors.pack_start(box_button,false,false,1);


		box_v1.set_orientation(Gtk::ORIENTATION_HORIZONTAL);
		box_v1.set_border_width(5);
		box_v1.set_spacing(30);

		box_v2.set_orientation(Gtk::ORIENTATION_HORIZONTAL);
		box_v2.set_border_width(5);
		box_v2.set_spacing(30);

		box_v3.set_orientation(Gtk::ORIENTATION_HORIZONTAL);
		box_v3.set_border_width(5);
		box_v3.set_spacing(30);

		box_button.set_orientation(Gtk::ORIENTATION_HORIZONTAL);
		box_button.set_border_width(5);
		box_button.set_spacing(30);

		box_parameters.set_border_width(5);
		box_parameters.set_spacing(30);

			//parameters_units - tree model columns
			parameters_units_list = Gtk::ListStore::create(parameters_units_columns);
			parameters_units_combobox.set_model(parameters_units_list);
			Gtk::TreeModel::Row parameters_units_rows = *(parameters_units_list->append());
			parameters_units_rows[parameters_units_columns.columns_id] = "alat";
			parameters_units_rows = *(parameters_units_list->append());
			parameters_units_rows[parameters_units_columns.columns_id] = "angstrom";
			parameters_units_rows = *(parameters_units_list->append());
			parameters_units_rows[parameters_units_columns.columns_id] = "bohr";

			parameters_units_combobox.pack_start(parameters_units_columns.columns_id);
			parameters_units_combobox.set_entry_text_column(parameters_units_columns.columns_id);
			parameters_units_combobox.set_active(1);

			parameters_units_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &parameters_units_value, parameters_units_columns.columns_id, &parameters_units_combobox, parameters_units_id, 1, "NULL" ));
			Gtk::Label *label_parameters_units = Gtk::manage(new Gtk::Label("Units for CELL_PARAMETERS: "));
			box_parameters.pack_start(*label_parameters_units,false,false,1);
			box_parameters.pack_start(parameters_units_combobox,false,false,1);

			//lattice vectors
			sbutton_v1_1.set_adjustment(adj_sb_v1_1);
			sbutton_v1_1.set_snap_to_ticks();
			sbutton_v1_1.set_numeric();
			sbutton_v1_1.set_digits(6);
			box_v1.pack_start(*label_v1_1,false,false,1);
			box_v1.pack_start(sbutton_v1_1,false,false,1);

			sbutton_v1_2.set_adjustment(adj_sb_v1_2);
			sbutton_v1_2.set_snap_to_ticks();
			sbutton_v1_2.set_numeric();
			sbutton_v1_2.set_digits(6);
			box_v1.pack_start(*label_v1_2,false,false,1);
			box_v1.pack_start(sbutton_v1_2,false,false,1);

			sbutton_v1_3.set_adjustment(adj_sb_v1_3);
			sbutton_v1_3.set_snap_to_ticks();
			sbutton_v1_3.set_numeric();
			sbutton_v1_3.set_digits(6);
			box_v1.pack_start(*label_v1_3,false,false,1);
			box_v1.pack_start(sbutton_v1_3,false,false,1);

			sbutton_v2_1.set_adjustment(adj_sb_v2_1);
			sbutton_v2_1.set_snap_to_ticks();
			sbutton_v2_1.set_numeric();
			sbutton_v2_1.set_digits(6);
			box_v2.pack_start(*label_v2_1,false,false,1);
			box_v2.pack_start(sbutton_v2_1,false,false,1);

			sbutton_v2_2.set_adjustment(adj_sb_v2_2);
			sbutton_v2_2.set_snap_to_ticks();
			sbutton_v2_2.set_numeric();
			sbutton_v2_2.set_digits(6);
			box_v2.pack_start(*label_v2_2,false,false,1);
			box_v2.pack_start(sbutton_v2_2,false,false,1);

			sbutton_v2_3.set_adjustment(adj_sb_v2_3);
			sbutton_v2_3.set_snap_to_ticks();
			sbutton_v2_3.set_numeric();
			sbutton_v2_3.set_digits(6);
			box_v2.pack_start(*label_v2_3,false,false,1);
			box_v2.pack_start(sbutton_v2_3,false,false,1);

			sbutton_v3_1.set_adjustment(adj_sb_v3_1);
			sbutton_v3_1.set_snap_to_ticks();
			sbutton_v3_1.set_numeric();
			sbutton_v3_1.set_digits(6);
			box_v3.pack_start(*label_v3_1,false,false,1);
			box_v3.pack_start(sbutton_v3_1,false,false,1);

			sbutton_v3_2.set_adjustment(adj_sb_v3_2);
			sbutton_v3_2.set_snap_to_ticks();
			sbutton_v3_2.set_numeric();
			sbutton_v3_2.set_digits(6);
			box_v3.pack_start(*label_v3_2,false,false,1);
			box_v3.pack_start(sbutton_v3_2,false,false,1);

			sbutton_v3_3.set_adjustment(adj_sb_v3_3);
			sbutton_v3_3.set_snap_to_ticks();
			sbutton_v3_3.set_numeric();
			sbutton_v3_3.set_digits(6);
			box_v3.pack_start(*label_v3_3,false,false,1);
			box_v3.pack_start(sbutton_v3_3,false,false,1);

			//button to update cell parameters
			button_update_cell_param.set_label("Update cell parameters");
			box_button.pack_end(button_update_cell_param,false,false,1);
			button_update_cell_param.signal_clicked().connect(sigc::mem_fun(*this, &window_main::update_cell_param) );

			//info about ibrav
			box_button.pack_start(*label_info_ibrav,false,false,1);



	//ATOMS TAB:
		//BOXES
		scrolled_box_atoms.set_hexpand();
		scrolled_box_atoms.set_vexpand();
		scrolled_box_atoms.add(box_main_atoms);
		scrolled_box_atoms.set_placement(Gtk::CORNER_TOP_LEFT);
		scrolled_box_atoms.set_shadow_type(Gtk::SHADOW_IN);

		box_main_atoms.set_orientation(Gtk::ORIENTATION_VERTICAL);
		box_main_atoms.set_border_width(20);
		box_main_atoms.set_spacing(10);
		box_main_atoms.pack_start(*label_atom_info,false,false,1);
		box_main_atoms.pack_start(frame_atoms,false,false,1);
		box_main_atoms.pack_start(box_positions_units,false,false,1);

		frame_atoms.add(box_atoms_in_frame);

		box_atoms_in_frame.set_border_width(10);
		box_atoms_in_frame.set_spacing(10);
		box_atoms_in_frame.set_orientation(Gtk::ORIENTATION_VERTICAL);
		box_atoms_in_frame.set_homogeneous();
		box_atoms_in_frame.add(box_atoms_labels);

		box_atoms_labels.set_border_width(10);
		box_atoms_labels.set_spacing(10);
		box_atoms_labels.set_orientation(Gtk::ORIENTATION_HORIZONTAL);
		box_atoms_labels.set_homogeneous();
		box_atoms_labels.pack_start(*label_atom_name,false,false,1);
		box_atoms_labels.pack_start(*label_atom_mass,false,false,1);
		box_atoms_labels.pack_start(*label_atom_pspot,false,false,1);
		box_atoms_labels.pack_start(*label_atom_num,false,false,1);
		box_atoms_labels.pack_start(*label_atom_empty,false,false,1);

		box_positions_units.set_border_width(5);
		box_positions_units.set_spacing(30);

			//positions_units - tree model columns
			positions_units_list = Gtk::ListStore::create(positions_units_columns);
			positions_units_combobox.set_model(positions_units_list);
			Gtk::TreeModel::Row positions_units_rows = *(positions_units_list->append());
			positions_units_rows[positions_units_columns.columns_id] = "alat";
			positions_units_rows = *(positions_units_list->append());
			positions_units_rows[positions_units_columns.columns_id] = "angstrom";
			positions_units_rows = *(positions_units_list->append());
			positions_units_rows[positions_units_columns.columns_id] = "bohr";
			positions_units_rows = *(positions_units_list->append());
			positions_units_rows[positions_units_columns.columns_id] = "crystal";
			positions_units_rows = *(positions_units_list->append());
			positions_units_rows[positions_units_columns.columns_id] = "crystal_sg";

			positions_units_combobox.pack_start(positions_units_columns.columns_id);
			positions_units_combobox.set_entry_text_column(positions_units_columns.columns_id);
			positions_units_combobox.set_active(1);

			positions_units_combobox.signal_changed().connect( sigc::bind<Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring,int,Glib::ustring>( sigc::mem_fun(*this,&window_main::changed_combobox), &positions_units_value, positions_units_columns.columns_id, &positions_units_combobox, positions_units_id, 1, "ATOMIC_POSITIONS" ));
			Gtk::Label *label_positions_units = Gtk::manage(new Gtk::Label("Units for ATOMIC_POSITIONS: "));
			box_positions_units.pack_start(*label_positions_units,false,false,1);
			box_positions_units.pack_start(positions_units_combobox,false,false,1);

			//atoms' specifications
			create_atoms_entries();


			button_update_species.set_label("Update ATOMIC_SPECIES");
			box_positions_units.pack_end(button_update_species,false,false,1);
			button_update_species.signal_clicked().connect(sigc::mem_fun(*this, &window_main::update_atomic_species) );


	//SCRIPT TAB
		//GRID
		Gtk::Grid *grid_script = Gtk::manage(new Gtk::Grid());
		grid_script->set_border_width(5);

		//SCROLL FIELD WITH ENTRY FIELD FOR WRITING SCRIPTS
		Gtk::Label *script_label = Gtk::manage(new Gtk::Label("SCRIPT EDITOR "));
		grid_script->attach(*script_label,2,0,2,1);

		grid_script->attach(script_scroll,0,1,6,6); //script_scroll is a ScrolledWindow object to put the TextView in
		script_scroll.set_hexpand();
		script_scroll.set_vexpand();
		script_scroll.add(script_field);
		script_scroll.set_placement(Gtk::CORNER_TOP_LEFT);
		script_scroll.set_shadow_type(Gtk::SHADOW_IN);

		Glib::RefPtr<Gtk::TextBuffer::Tag> refTagMatch = Gtk::TextBuffer::Tag::create();
		Glib::RefPtr<Gtk::TextBuffer::TagTable> refTagTable = Gtk::TextBuffer::TagTable::create();
		refTagTable->add(refTagMatch);
		script_buffer = Gtk::TextBuffer::create(refTagTable);
		script_field.set_buffer(script_buffer);
		set_script_buffer();


		//BUTTONS
		//save entry field
		save_script_button.set_label("Save");
		grid_script->attach(save_script_button,2,7,1,1);
		save_script_button.signal_clicked().connect(sigc::mem_fun(*this, &window_main::click_on_save_script) );
		//clear entry field
		clear_script_button.set_label("Clear");
		grid_script->attach_next_to(clear_script_button,save_script_button, Gtk::POS_RIGHT, 1,2);
		clear_script_button.signal_clicked().connect(sigc::mem_fun(*this, &window_main::click_on_clear_script) );

		grid_script->set_column_homogeneous(true);


	//NOTEBOOK
	pages.append_page(*grid_control,"Control");
	pages.append_page(*grid_system,"System");
	pages.append_page(*grid_electrons,"Electrons");
	pages.append_page(*grid_ions,"Ions");
	pages.append_page(*grid_cell,"Cell");
	pages.append_page(box_cell_parameters,"Cell parameters");
	pages.append_page(scrolled_box_atoms,"Atoms");
	pages.append_page(*grid_script,"Script");
	mbox->add(pages);
	pages.set_hexpand();
	pages.set_vexpand();


	mbox->show_all(); //necessary to show items in a mbox
	update_atoms_fields();
	help_link1.set_visible(false);
	help_link2.set_visible(false);

}

window_main::~window_main()
{
//deconstructor
}

////////////////////////////////////////////////////////////////////////////////

bool window_main:: delete_event(GdkEventAny* any_event)
{
	click_on_quit();
	return FALSE;
}


//////////////// MENU ITEMS' FUNCTIONS /////////////////////////////////////////

void window_main:: click_on_open_script()
{
    Gtk::FileChooserDialog dialog("Please choose a script to open: ", Gtk::FILE_CHOOSER_ACTION_OPEN);
    dialog.set_transient_for(*this);

    dialog.add_button("_Cancel", Gtk::RESPONSE_CANCEL);
    dialog.add_button("_Select", Gtk::RESPONSE_OK);

    int result = dialog.run();

    switch(result)
    {
    	case(Gtk::RESPONSE_OK):
      {
				script_to_open = dialog.get_filename();
				break;
      }
	    case(Gtk::RESPONSE_CANCEL):
	    {
	    	break;
			}
			default:
			{
				break;
			}
		}

		set_script_buffer(script_to_open);
}


void window_main:: click_on_set_basic_script()
{
	Gtk::FileChooserDialog chooser_dialog("Please choose a script to be set as basic script: ", Gtk::FILE_CHOOSER_ACTION_OPEN);
    chooser_dialog.set_transient_for(*this);

    chooser_dialog.add_button("_Cancel", Gtk::RESPONSE_CANCEL);
    chooser_dialog.add_button("_Select", Gtk::RESPONSE_OK);

    int result = chooser_dialog.run();
	string nfile;

    switch(result)
    {
    	case(Gtk::RESPONSE_OK):
		{
			nfile = chooser_dialog.get_filename();
			break;
		}
	    case(Gtk::RESPONSE_CANCEL):
	    {
	    	break;
		}
		default:
		{
			break;
		}
	}

	set_script_buffer(nfile);
	ifstream new_basic_script(nfile);
	fstream old_basic_script("basic_script.in");
	old_basic_script << new_basic_script.rdbuf();
	new_basic_script.close();
	old_basic_script.close();
	string name = nfile.substr(nfile.find_last_of("/")+1);
	Gtk::MessageDialog info_dialog("'"+name+"' is set as a basic script now. If you want to go back to default script use this function again and choose 'basic_script.txt'.", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
	info_dialog.add_button("OK", Gtk::RESPONSE_OK);
	info_dialog.set_transient_for(*this);
	info_dialog.run();

}


void window_main:: click_on_quit()
{
	Gtk::MessageDialog dialog("Do you want to save current script before closing the aplication?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
	dialog.add_button("Yes", Gtk::RESPONSE_OK);
	dialog.add_button("No", Gtk::RESPONSE_CLOSE);
	dialog.set_transient_for(*this);
	int result = dialog.run();
	switch(result)
	{
		case(Gtk::RESPONSE_OK):
		{
			click_on_save_script();
			break;
		}
		case(Gtk::RESPONSE_CANCEL):
		{
			break;
		}
		default:
		{
			break;
		}

	}
	hide();
}


void window_main:: click_on_choose_folder()
{
	Gtk::FileChooserDialog dialog("Please choose a default folder for your results: ", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
	dialog.set_transient_for(*this);

	dialog.add_button("_Cancel", Gtk::RESPONSE_CANCEL);
	dialog.add_button("_Select", Gtk::RESPONSE_OK);

	int result = dialog.run();

	switch(result)
	{
		case(Gtk::RESPONSE_OK):
		{
			new_folder = dialog.get_filename();
			break;
		}
		case(Gtk::RESPONSE_CANCEL):
		{
			break;
		}
		default:
		{
			break;
		}
	}

	Glib::ustring txt = script_buffer->get_text();
	update_script("pseudo_dir", "'"+new_folder+"'",0,"&CONTROL\n");

	//updating "basic_script.in" for a permament change
	ifstream istr;
	istr.open("basic_script.in");
	ofstream ostr;
	ostr.open("t.in");
	try
	{
		string new_script = "";
		string line;
		while (istr)
		{
			getline(istr,line);
			if (line.find("pseudo_dir = ") == 0)
			{
				line = "pseudo_dir = '"+new_folder+"'";
			}
			new_script = new_script + line + "\n";
		}
		new_script.erase(new_script.end()-1);
		ostr << new_script;
	;}
	catch(...)
	{
		error_message("An error occurred while changing default folder in file 'basic_script.in'.");
	}
	istr.close();
	ostr.close();
	rename("t.in","basic_script.in");

}

void window_main:: click_on_help1()
{
	help_link1.clicked();
}

void window_main:: click_on_help2()
{
	help_link2.clicked();
}

////////////////////////////////////////////////////////////////////////////////

void window_main:: changed_combobox(Glib::ustring *chosen_value, Gtk::TreeModelColumn<Glib::ustring> col_id, Gtk::ComboBox* combo, Glib::ustring id, int update_type, Glib::ustring values_tab)
{
	Gtk::TreeModel::iterator iter = combo->get_active();
	Gtk::TreeModel::Row row = *iter;
	Glib::ustring recent_value = *chosen_value;
	*chosen_value = row[col_id];
	if (update_type == 0)
	{update_script(id, *chosen_value, 0, values_tab);}
	else if (update_type == 1)
	{update_script(id, id+" "+*chosen_value, 1, values_tab);}
	if (id == "ibrav")
	{
		if (*chosen_value == "0")
		{
			sbutton_v1_1.set_editable(true);
			sbutton_v1_2.set_editable(true);
			sbutton_v1_3.set_editable(true);
			sbutton_v2_1.set_editable(true);
			sbutton_v2_2.set_editable(true);
			sbutton_v2_3.set_editable(true);
			sbutton_v3_1.set_editable(true);
			sbutton_v3_2.set_editable(true);
			sbutton_v3_3.set_editable(true);
			button_update_cell_param.set_visible(true);
		}
		else
		{
			sbutton_v1_1.set_editable(false);
			sbutton_v1_2.set_editable(false);
			sbutton_v1_3.set_editable(false);
			sbutton_v2_1.set_editable(false);
			sbutton_v2_2.set_editable(false);
			sbutton_v2_3.set_editable(false);
			sbutton_v3_1.set_editable(false);
			sbutton_v3_2.set_editable(false);
			sbutton_v3_3.set_editable(false);
			button_update_cell_param.set_visible(false);
		}
	}
}


void window_main:: update_script(Glib::ustring id, Glib::ustring chosen, int update_type, Glib::ustring values_tab)
/*update type:
 0 -- old value of combobox -> new value of combobox
 1 -- old line -> new line
 2 -- old block of text -> new block of text
*/
{
	Glib::ustring script_text = script_buffer->get_text();
	try
	{
		Glib::ustring old_thing;
		Glib::ustring new_thing;

		if (update_type == 0)
		{
			old_thing = script_text.substr( script_text.find(id), script_text.substr(script_text.find(id)).find("\n") );
			new_thing = id+" = "+chosen;
		}
		else if(update_type == 1)
		{
			old_thing = script_text.substr( script_text.find(id), script_text.substr(script_text.find(id)).find("\n") );
			new_thing = chosen;
		}
		else if(update_type == 2)
		{
			old_thing = id;
			new_thing = chosen;
		}
		script_text.replace(script_text.rfind(old_thing), old_thing.length(), new_thing);
		script_buffer->set_text(script_text);

	;}
	catch(...)
	{
		if (update_type == 0)
		{
			add_to_script(id, values_tab, id+" = "+chosen);
		}
		else if(update_type == 1)
		{
			add_to_script(id, "" ,values_tab+"\n"+chosen);
		}
		else
		{
			error_message("An error occurred. This value might not be in the script. Add it or clear the script. ");
		}
	}
	save_temp();
}

//////////////////////////// SCRIPT FUNCTIONS //////////////////////////////////

void window_main:: set_script_buffer(Glib::ustring script_filename)
{
	ifstream str;
	str.open(script_filename);

	try
	{
		string line;
		string buffer = "";
		while(str)
		{
			getline(str,line);
			buffer = buffer + line + "\n";
		}
		script_buffer->set_text(Glib::convert (buffer, "UTF-8", "ISO8859-1"));
    }
	catch(...)
	{
		error_message("An error occurred while opening 'basic_script.in'.");
	}
	str.close();

}


void window_main:: click_on_clear_script()
{
	Gtk::MessageDialog dialog("Do you want to clear current script?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
	auto yes_button = dialog.add_button("Yes", Gtk::RESPONSE_CLOSE);
	auto no_button = dialog.add_button("No", Gtk::RESPONSE_CLOSE);
	yes_button->signal_clicked().connect(sigc::mem_fun(*this, &window_main::click_on_yes_clear_script) );
	dialog.set_transient_for(*this);
	dialog.run();
}


void window_main:: click_on_yes_clear_script()
{
	calculation_combobox.set_active(1);
	verbosity_combobox.set_active(1);
	restart_mode_combobox.set_active(1);
	disk_io_combobox.set_active(1);
	occupations_combobox.set_active(1);
	smearing_combobox.set_active(1);
	exxdiv_treatment_combobox.set_active(1);
	U_projection_type_combobox.set_active(1);
	constrained_magnetization_combobox.set_active(1);
	assume_isolated_combobox.set_active(1);
	esm_bc_combobox.set_active(1);
	vdw_corr_combobox.set_active(1);
	mixing_mode_combobox.set_active(1);
	diagonalization_combobox.set_active(1);
	efield_phase_combobox.set_active(1);
	startingpot_combobox.set_active(1);
	startingwfc_combobox.set_active(1);
	ion_positions_combobox.set_active(1);
	pot_extrapolation_combobox.set_active(1);
	wfc_extrapolation_combobox.set_active(1);
	ion_temperature_combobox.set_active(1);
	cell_dofree_combobox.set_active(1);
	set_script_buffer();
}


void window_main:: click_on_save_script()
{
	Gtk::FileChooserDialog dialog("Please choose a folder", Gtk::FILE_CHOOSER_ACTION_SAVE);
    dialog.set_transient_for(*this);

    dialog.add_button("_Cancel", Gtk::RESPONSE_CANCEL);
    dialog.add_button("Select", Gtk::RESPONSE_OK);
    dialog.set_do_overwrite_confirmation(TRUE);

	string filename = "";
    int result = dialog.run();

    switch(result)
    {
        case(Gtk::RESPONSE_OK):
        {
            filename =  dialog.get_filename();
			if (is_filname_ok(filename) == 1)
			{
				try
				{
					ofstream output;
					output.open(filename);
					output << script_buffer->get_text() << endl;
					output.close();
					break;
				}
				catch(...)
				{
					error_message("An error occured while opening "+filename+".");
				}
			}
			else
			{
				error_message("Incorret path or name of the file!");
			}
        }
        case(Gtk::RESPONSE_CANCEL):
        {
            break;
        }
        default:
        {
            break;
        }
    }

}


int window_main:: is_filname_ok(string fname)
{
	if (fname.length() > 3)
	{return(fname.substr(fname.length()-3,3) == ".in");}
	else
	{return 0;}
}


void window_main:: error_message(Glib::ustring message)
{
    Gtk::MessageDialog dialog(message, false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE, true);
    dialog.set_transient_for(*this);
    dialog.run();
}

void window_main:: update_cell_param()
{
	string script_text = script_buffer->get_text();
	try
	{
		Glib::ustring old_parameters = script_text.substr( script_text.find("CELL_PARAMETERS"), script_text.length() );
		int l_end = 0;
		int pos;
		for(int i = 0; i != old_parameters.length(); i++)
		{
			if (l_end == 4)
			{
				pos = i;
				break;
			}
			else if( isalpha(old_parameters[i]) and l_end>1)
			{
				pos = i - 1;
				break;
			}
			if (old_parameters[i] == '\n')
			{
				l_end = l_end + 1;
			}
		}
		old_parameters = old_parameters.substr(0,pos);
		Glib::ustring new_parameters = old_parameters.substr(0,old_parameters.find('\n')+1)+" "+to_string(sbutton_v1_1.get_value())+" "+to_string(sbutton_v1_2.get_value())+" "+to_string(sbutton_v1_3.get_value())+"\n "+to_string(sbutton_v2_1.get_value())+" "+to_string(sbutton_v2_2.get_value())+" "+to_string(sbutton_v2_3.get_value())+"\n "+to_string(sbutton_v3_1.get_value())+" "+to_string(sbutton_v3_2.get_value())+" "+to_string(sbutton_v3_3.get_value())+"\n";
		update_script(old_parameters, new_parameters, 2, "NULL");

	}
	catch(...)
	{
		Glib::ustring new_parameters = "CELL_PARAMETERS "+parameters_units_value+"\n"+" "+to_string(sbutton_v1_1.get_value())+" "+to_string(sbutton_v1_2.get_value())+" "+to_string(sbutton_v1_3.get_value())+"\n "+to_string(sbutton_v2_1.get_value())+" "+to_string(sbutton_v2_2.get_value())+" "+to_string(sbutton_v2_3.get_value())+"\n "+to_string(sbutton_v3_1.get_value())+" "+to_string(sbutton_v3_2.get_value())+" "+to_string(sbutton_v3_3.get_value())+"\n";
		add_to_script("'CELL_PARAMETERS'", "/\n", new_parameters);
	}
}

void window_main:: changed_ntyp()
{
	sbutton_nat.set_range(sbutton_ntyp.get_value(),100);
	update_atoms_fields();
	check_if_nat_ok("ntyp");
	update_script("ntyp", to_string(int(sbutton_ntyp.get_value())), 0, "&SYSTEM\n");
}

void window_main:: changed_nat()
{
	update_script("nat", to_string(int(sbutton_nat.get_value())), 0, "&SYSTEM\n");
	check_if_nat_ok("nat");
}

void window_main:: add_to_script(Glib::ustring name, Glib::ustring where, Glib::ustring what)
{
	Gtk::MessageDialog dialog(name+" is not in the script. Do you want to add it?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
	dialog.add_button("Yes", Gtk::RESPONSE_OK);
	dialog.add_button("No", Gtk::RESPONSE_CLOSE);
	dialog.set_transient_for(*this);
	int result = dialog.run();
	switch(result)
	{
		case(Gtk::RESPONSE_OK):
		{
			try
			{
				update_script(where,where+what+"\n",2,"NULL")
			;}
			catch(...)
			{
				error_message("An error occurred while adding value to script.");
			}
			break;
		}
		case(Gtk::RESPONSE_CANCEL):
		{
			break;
		}
		default:
		{
			break;
		}
	}
}
void window_main:: create_atoms_entries()
{
	for(int i = 0; i<40; i++)
	{
		atoms_names_entries.push_back(Gtk::manage(new Gtk::Entry()));
		atoms_mass_sbuttons.push_back(Gtk::manage(new Gtk::SpinButton()));
		atoms_mass_adj.push_back(Gtk::Adjustment::create(1.0, 1.0, 400.0, 0.00001, 100.0, 0.0));
		atoms_pspot_entries.push_back(Gtk::manage(new Gtk::Entry()));
		atoms_num_sbuttons.push_back(Gtk::manage(new Gtk::SpinButton()));
		atoms_num_adj.push_back(Gtk::Adjustment::create(1.0, 1.0, 100.0, 1.0, 100.0, 0.0));
		atoms_buttons.push_back(Gtk::Button());
		atoms_boxes.push_back(Gtk::manage(new Gtk::Box()));
	}
	for(int i = 0; i<atoms_boxes.size();i++)
	{
		atoms_boxes[i]->set_orientation(Gtk::ORIENTATION_HORIZONTAL);
		atoms_boxes[i]->set_border_width(10);
		atoms_boxes[i]->set_spacing(10);
		atoms_boxes[i]->set_homogeneous();

		atoms_mass_sbuttons[i]->set_adjustment(atoms_mass_adj[i]);
		atoms_mass_sbuttons[i]->set_snap_to_ticks();
		atoms_mass_sbuttons[i]->set_numeric();
		atoms_mass_sbuttons[i]->set_digits(5);
		atoms_num_sbuttons[i]->set_adjustment(atoms_num_adj[i]);
		atoms_num_sbuttons[i]->set_snap_to_ticks();
		atoms_num_sbuttons[i]->set_numeric();
		atoms_num_sbuttons[i]->set_digits(0);
		atoms_num_sbuttons[i]->signal_value_changed().connect(sigc::bind<string> (sigc::mem_fun(*this, &window_main::check_if_nat_ok), "click"));

		atoms_buttons[i].set_label("Add positions");
		atoms_buttons[i].signal_clicked().connect(sigc::bind<int> (sigc::mem_fun(*this, &window_main::add_atomic_pos), i));

		atoms_boxes[i]->add(*atoms_names_entries[i]);
		atoms_boxes[i]->add(*atoms_mass_sbuttons[i]);
		atoms_boxes[i]->add(*atoms_pspot_entries[i]);
		atoms_boxes[i]->add(*atoms_num_sbuttons[i]);
		atoms_boxes[i]->add(atoms_buttons[i]);
		box_atoms_in_frame.add(*atoms_boxes[i]);
	}
}

void window_main:: update_atoms_fields()
{
	int i_max = sbutton_ntyp.get_value();
	for(int i = 0; i<atoms_names_entries.size();i++)
	{
		if (i<i_max)
		{
			atoms_boxes[i]->set_visible(true);
		}
		else
		{
			atoms_boxes[i]->set_visible(false);
		}
	}
}

void window_main:: update_atomic_species()
{
	string script_text = script_buffer->get_text();
	Glib::ustring new_species = "ATOMIC_SPECIES\n";
	int tot = 0;
	for(int i = 0; i<atoms_boxes.size();i++)
	{
		if(atoms_boxes[i]->get_visible() == 1)
		{
			if (atoms_names_entries[i]->get_text() == "" or atoms_pspot_entries[i]->get_text() == "")
			{
				error_message("At least one of entry fields is empty. Please enter all values.");
				return;
			}
			tot = tot + int(atoms_num_sbuttons[i]->get_value());
			new_species = new_species + atoms_names_entries[i]->get_text() + " " + to_string(atoms_mass_sbuttons[i]->get_value()) + " " + atoms_pspot_entries[i]->get_text() + "\n";
		}
	}
	if (int(sbutton_nat.get_value()) != tot)
	{
		error_message("The sum of atoms is not correct! (nat = "+to_string(int(sbutton_nat.get_value()))+" and the sum is "+to_string(tot) );
		return;
	}
	try
	{
		Glib::ustring old_species = script_text.substr( script_text.find("ATOMIC_SPECIES"), script_text.length() );
		Glib::ustring possibilities [] = {"ATOMIC_POSITIONS","K_POINTS","CELL_PARAMETERS","CONSTRAINTS","OCCUPATIONS","ATOMIC_FORCES"};

		int pos = old_species.find(possibilities[0]);
		for(int i = 0; i<6; i++)
		{
			auto a = old_species.find(possibilities[i]);
			if (a<pos)
			{
				pos = a;
			}
		}
		old_species = old_species.substr(0,pos);
		update_script(old_species, new_species+"\n", 2, "NULL");
	}
	catch(...)
	{
		add_to_script("'ATOMIC_SPECIES'", "/\n", new_species);
	}
}

void window_main:: check_if_nat_ok(string typ)
{
	int total_nat = 0;
	for(int i = 0; i<atoms_boxes.size(); i++)
	{
		if(atoms_boxes[i]->get_visible() == 1)
		{
			total_nat = total_nat + atoms_num_sbuttons[i]->get_value();
		}
	}
	if(total_nat == sbutton_nat.get_value())
	{
		if(typ=="click")
		{error_message("You have reached the maximum number of atoms (nat = "+to_string(int(sbutton_nat.get_value()))+").");}
		for(int i = 0; i<atoms_boxes.size(); i++)
		{
			atoms_num_sbuttons[i]->set_range(1,atoms_num_sbuttons[i]->get_value());
		}
	}
	else if(total_nat > sbutton_nat.get_value())
	{
		for(int i = atoms_boxes.size()-1; i>=0; i--)
		{
			int now = atoms_num_sbuttons[i]->get_value();
			int new_range = now - (total_nat - sbutton_nat.get_value());
			if (new_range < 1)
			{
				atoms_num_sbuttons[i]->set_range(1,1);
				total_nat = total_nat - (now - 1);
			}
			else
			{
				atoms_num_sbuttons[i]->set_range(1,new_range);
				break;
			}
		}
	}
	else
	{
		for(int i = 0; i<atoms_boxes.size(); i++)
		{
			atoms_num_sbuttons[i]->set_range(1,100);
		}
	}
}

void window_main:: save_temp()
{
	ofstream temp_file;
	temp_file.open("temp_script.in");
	string script_text = script_buffer->get_text();
	temp_file << script_text;
	temp_file.close();
}

void window_main:: add_atomic_pos(int ind)
{
	string atom_name = atoms_names_entries[ind]->get_text();
	int no_atoms = atoms_num_sbuttons[ind]->get_value();
	if(atom_name == "")
	{
		error_message("Please enter atom's name before adding postions.");
		return;
	}
	int how_many = how_many_atoms(atom_name+" ");
	if(how_many == 0)
	{
		aap_win.set_params(atom_name,no_atoms);
		option_add_atom = 0;
	}
	else if(how_many >= no_atoms)
	{
		Gtk::MessageDialog dialog("There are already "+ to_string(how_many) + " atom's postions for atom " + atom_name +". Do you want to replace all of them?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
		dialog.add_button("Replace", 2);
		dialog.add_button("Cancel", 0);
		dialog.set_transient_for(*this);
		int result = dialog.run();
		switch(result)
		{
			case(2):
			{
				aap_win.set_params(atom_name,no_atoms);
				option_add_atom = 2;
				break;
			}
			case(0):
			{
				return;
			}
			default:
			{
				break;
			}
		}
	}
	else
	{
		Gtk::MessageDialog dialog("There are already "+ to_string(how_many) + " atom's postions for atom " + atom_name +". Do you want to add extra values or replace all of them?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
		dialog.add_button("Add extra", 1);
		dialog.add_button("Replace", 2);
		dialog.add_button("Cancel", 0);
		dialog.set_transient_for(*this);
		int result = dialog.run();
		switch(result)
		{
			case(1):
			{
				aap_win.set_params(atom_name,no_atoms-how_many);
				option_add_atom = 1;
				break;
			}
			case(2):
			{
				aap_win.set_params(atom_name,no_atoms);
				option_add_atom = 2;
				break;
			}
			case(0):
			{
				return;
			}
			default:
			{
				break;
			}
		}
	}
	aap_win.show();
	aap_win.signal_hide().connect(sigc::bind<string> (sigc::mem_fun(*this, &window_main::add_atom_pos_to_script), atom_name));
	signal_times = 0;
}

void window_main:: add_atom_pos_to_script(string name)
{
	if(signal_times==0)
	{
		Glib::ustring txt = script_buffer->get_text();
		Glib::ustring from_win = aap_win.final_txt;
		if(option_add_atom == 0 or option_add_atom == 1) //add new or extra
		{
			try
			{
				Glib::ustring old_txt = txt.substr( txt.find("ATOMIC_POSITIONS"), txt.length() );
				old_txt = old_txt.substr(0, old_txt.find("\n")+1);
				update_script(old_txt, old_txt+from_win, 2, "NULL");
			}
			catch(...)
			{
				Glib::ustring new_txt = "ATOMIC_POSITIONS "+positions_units_value+"\n"+from_win+"\n";
				add_to_script("'ATOMIC_POSITIONS'", "/\n", new_txt);
			}
		}
		else if(option_add_atom == 2) //replace
		{
			remove_old_pos(name, txt);
			Glib::ustring old_txt = txt.substr( txt.find("ATOMIC_POSITIONS"), txt.length() );
			old_txt = old_txt.substr(0,old_txt.find("\n")+1);
			update_script(old_txt, old_txt + from_win, 2, "NULL");
		}
		signal_times++;
	}
	else
	{
		return ;
	}
}

int window_main:: how_many_atoms(string name)
{
	Glib::ustring txt = script_buffer->get_text();
	int result = 0;
	try
	{
		Glib::ustring text_to_search = txt.substr(txt.find("ATOMIC_POSITIONS")+17, txt.length());
		Glib::ustring possibilities [] = {"K_POINTS","CELL_PARAMETERS","CONSTRAINTS","OCCUPATIONS","ATOMIC_FORCES"};
		int pos = text_to_search.find(possibilities[0]);
		for(int i = 1; i<5; i++)
		{
			auto a = text_to_search.find(possibilities[i]);
			if (a<pos)
			{
				pos = a;
			}

		}
		text_to_search = text_to_search.substr(0,pos);
		while(1)
		{
			int j = text_to_search.find(name);
			if(j < 0){break;}
			else
			{
				result++;
				text_to_search = text_to_search.substr(j+name.length(),text_to_search.length());
			}
		}
		return result;

	;}
	catch(...)
	{
		return 0;
	}
}

void window_main:: remove_old_pos(string name, string txt)
{
	string old = txt;
	while(1)
	{
		int pos1 = txt.find(name);
		if(pos1 < 0){break;}
		else
		{
			string p = txt.substr(pos1, txt.length());
			p = p.substr(0,p.find("\n"));
			txt.erase(pos1, p.length());
		}
	}
	update_script(old,txt,2,"NULL");
}
