/*
 * Copyright 2019 Oliwia Chrapowicka
 *
 * This file is part of QESG.
 *
 * QESG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * QESG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with QESG.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef window_main_H
#define window_main_H

#include <iostream>
#include <fstream>
#include<gtkmm.h>
#include <cstdio>
#include <vector>
#include "add_atomic_pos.cpp"

using namespace std;


class window_main : public Gtk::Window
{
	public:
		window_main(int,int,Glib::ustring);
		virtual ~window_main();

	protected:
		bool delete_event(GdkEventAny*);
		void click_on_open_script();
		void click_on_set_basic_script();
		void click_on_quit();
		void click_on_choose_folder();
		void click_on_help1();
		void click_on_help2();
		void changed_combobox(Glib::ustring*,Gtk::TreeModelColumn<Glib::ustring>,Gtk::ComboBox*,Glib::ustring, int, Glib::ustring);
		void update_script(Glib::ustring,Glib::ustring, int, Glib::ustring);
		void set_script_buffer(Glib::ustring = "basic_script.in");
		void click_on_clear_script();
		void click_on_yes_clear_script();
		void click_on_save_script();
		int is_filname_ok(string);
		void error_message(Glib::ustring);
		void update_cell_param();
		void add_cell_param();
		void changed_ntyp();
		void changed_nat();
		void add_to_script(Glib::ustring, Glib::ustring, Glib::ustring);
		void create_atoms_entries();
		void update_atoms_fields();
		void update_atomic_species();
		void check_if_nat_ok(string);
		void save_temp();
		void add_atom_pos_to_script(string);
		int how_many_atoms(string);
		void remove_old_pos(string, string);
		void add_atomic_pos(int);

		//List class
		class ModelColumns : public Gtk::TreeModel::ColumnRecord
		{
			public:
				ModelColumns()
    			{ add(columns_id);}
    			Gtk::TreeModelColumn<Glib::ustring> columns_id;
  		};

		//CHILD WIDGETS:
		Gtk::Notebook pages;
		Glib::ustring new_folder;
		Glib::ustring script_to_open;
		Gtk::LinkButton help_link1;
		Gtk::LinkButton help_link2;


		//&CONTROL tab
			//calculation
			ModelColumns calculation_columns;
			Gtk::ComboBox calculation_combobox;
			Glib::RefPtr<Gtk::ListStore> calculation_list;
			Glib::ustring calculation_value = "scf";
			Glib::ustring calculation_id = "calculation";
			//verbosity
			ModelColumns verbosity_columns;
			Gtk::ComboBox verbosity_combobox;
			Glib::RefPtr<Gtk::ListStore> verbosity_list;
			Glib::ustring verbosity_value = "low";
			Glib::ustring verbosity_id = "verbosity";
			//restart_mode
			ModelColumns restart_mode_columns;
			Gtk::ComboBox restart_mode_combobox;
			Glib::RefPtr<Gtk::ListStore> restart_mode_list;
			Glib::ustring restart_mode_value = "from_scratch";
			Glib::ustring restart_mode_id = "restart_mode";
			//disk_io
			ModelColumns disk_io_columns;
			Gtk::ComboBox disk_io_combobox;
			Glib::RefPtr<Gtk::ListStore> disk_io_list;
			Glib::ustring disk_io_value = "low";
			Glib::ustring disk_io_id = "disk_io";


		//SYSTEM tab
			//nat
			Gtk::Label *label_nat = Gtk::manage(new Gtk::Label("nat = "));
			Glib::RefPtr<Gtk::Adjustment> adj_sb_nat = Gtk::Adjustment::create(1.0, 1, 100.0, 1.0, 100.0, 0.0);
			Gtk::SpinButton sbutton_nat;
			//nat
			Gtk::Label *label_ntyp = Gtk::manage(new Gtk::Label("ntyp = "));
			Glib::RefPtr<Gtk::Adjustment> adj_sb_ntyp = Gtk::Adjustment::create(1.0, 1, 40.0, 1.0, 100.0, 0.0);
			Gtk::SpinButton sbutton_ntyp;
			//ibrav
			ModelColumns ibrav_columns;
			Gtk::ComboBox ibrav_combobox;
			Glib::RefPtr<Gtk::ListStore> ibrav_list;
			Glib::ustring ibrav_value = "0";
			Glib::ustring ibrav_id = "ibrav";
			//occupations
			ModelColumns occupations_columns;
			Gtk::ComboBox occupations_combobox;
			Glib::RefPtr<Gtk::ListStore> occupations_list;
			Glib::ustring occupations_value = "smearing";
			Glib::ustring occupations_id = "occupations";
			//smearing
			ModelColumns smearing_columns;
			Gtk::ComboBox smearing_combobox;
			Glib::RefPtr<Gtk::ListStore> smearing_list;
			Glib::ustring smearing_value = "gaussian";
			Glib::ustring smearing_id = "smearing";
			//exxdiv_treatment
			ModelColumns exxdiv_treatment_columns;
			Gtk::ComboBox exxdiv_treatment_combobox;
			Glib::RefPtr<Gtk::ListStore> exxdiv_treatment_list;
			Glib::ustring exxdiv_treatment_value = "gygi-baldereschi";
			Glib::ustring exxdiv_treatment_id = "exxdiv_treatment";
			//U_projection_type
			ModelColumns U_projection_type_columns;
			Gtk::ComboBox U_projection_type_combobox;
			Glib::RefPtr<Gtk::ListStore> U_projection_type_list;
			Glib::ustring U_projection_type_value = "atomic";
			Glib::ustring U_projection_type_id = "U_projection_type";
			//constrained_magnetization
			ModelColumns constrained_magnetization_columns;
			Gtk::ComboBox constrained_magnetization_combobox;
			Glib::RefPtr<Gtk::ListStore> constrained_magnetization_list;
			Glib::ustring constrained_magnetization_value = "none";
			Glib::ustring constrained_magnetization_id = "constrained_magnetization";
			//assume_isolated
			ModelColumns assume_isolated_columns;
			Gtk::ComboBox assume_isolated_combobox;
			Glib::RefPtr<Gtk::ListStore> assume_isolated_list;
			Glib::ustring assume_isolated_value = "none";
			Glib::ustring assume_isolated_id = "assume_isolated";
			//esm_bc
			ModelColumns esm_bc_columns;
			Gtk::ComboBox esm_bc_combobox;
			Glib::RefPtr<Gtk::ListStore> esm_bc_list;
			Glib::ustring esm_bc_value = "pbc";
			Glib::ustring esm_bc_id = "esm_bc";
			//vdw_corr
			ModelColumns vdw_corr_columns;
			Gtk::ComboBox vdw_corr_combobox;
			Glib::RefPtr<Gtk::ListStore> vdw_corr_list;
			Glib::ustring vdw_corr_value = "none";
			Glib::ustring vdw_corr_id = "vdw_corr";

		//ELECTRONS tab
			//mixing_mode
			ModelColumns mixing_mode_columns;
			Gtk::ComboBox mixing_mode_combobox;
			Glib::RefPtr<Gtk::ListStore> mixing_mode_list;
			Glib::ustring mixing_mode_value = "plain";
			Glib::ustring mixing_mode_id = "mixing_mode";
			//diagonalization
			ModelColumns diagonalization_columns;
			Gtk::ComboBox diagonalization_combobox;
			Glib::RefPtr<Gtk::ListStore> diagonalization_list;
			Glib::ustring diagonalization_value = "david";
			Glib::ustring diagonalization_id = "diagonalization";
			//efield_phase
			ModelColumns efield_phase_columns;
			Gtk::ComboBox efield_phase_combobox;
			Glib::RefPtr<Gtk::ListStore> efield_phase_list;
			Glib::ustring efield_phase_value = "none";
			Glib::ustring efield_phase_id = "efield_phase";
			//startingpot
			ModelColumns startingpot_columns;
			Gtk::ComboBox startingpot_combobox;
			Glib::RefPtr<Gtk::ListStore> startingpot_list;
			Glib::ustring startingpot_value = "atomic";
			Glib::ustring startingpot_id = "startingpot";
			//startingwfc
			ModelColumns startingwfc_columns;
			Gtk::ComboBox startingwfc_combobox;
			Glib::RefPtr<Gtk::ListStore> startingwfc_list;
			Glib::ustring startingwfc_value = "atomic+random";
			Glib::ustring startingwfc_id = "startingwfc";

		//IONS tab
			//ion_positions
			ModelColumns ion_positions_columns;
			Gtk::ComboBox ion_positions_combobox;
			Glib::RefPtr<Gtk::ListStore> ion_positions_list;
			Glib::ustring ion_positions_value = "default";
			Glib::ustring ion_positions_id = "ion_positions";
			//pot_extrapolation
			ModelColumns pot_extrapolation_columns;
			Gtk::ComboBox pot_extrapolation_combobox;
			Glib::RefPtr<Gtk::ListStore> pot_extrapolation_list;
			Glib::ustring pot_extrapolation_value = "atomic";
			Glib::ustring pot_extrapolation_id = "pot_extrapolation";
			//wfc_extrapolation
			ModelColumns wfc_extrapolation_columns;
			Gtk::ComboBox wfc_extrapolation_combobox;
			Glib::RefPtr<Gtk::ListStore> wfc_extrapolation_list;
			Glib::ustring wfc_extrapolation_value = "none";
			Glib::ustring wfc_extrapolation_id = "wfc_extrapolation";
			//ion_temperature
			ModelColumns ion_temperature_columns;
			Gtk::ComboBox ion_temperature_combobox;
			Glib::RefPtr<Gtk::ListStore> ion_temperature_list;
			Glib::ustring ion_temperature_value = "not_controlled";
			Glib::ustring ion_temperature_id = "ion_temperature";

		//CELL tab
			//cell_dofree
			ModelColumns cell_dofree_columns;
			Gtk::ComboBox cell_dofree_combobox;
			Glib::RefPtr<Gtk::ListStore> cell_dofree_list;
			Glib::ustring cell_dofree_value = "all";
			Glib::ustring cell_dofree_id = "cell_dofree";

		//CELL_PARAMETERES tab
			Gtk::Box box_cell_parameters, box_vectors, box_v1, box_v2, box_v3, box_button, box_parameters;
			Gtk::Frame frame_vectors;
			Gtk::Button button_update_cell_param;
			Gtk::Label *label_info_ibrav = Gtk::manage(new Gtk::Label("Cell paramteres are optional, needed only if ibrav == 0 is specified, ignored otherwise."));

			//parameters_units
			ModelColumns parameters_units_columns;
			Gtk::ComboBox parameters_units_combobox;
			Glib::RefPtr<Gtk::ListStore> parameters_units_list;
			Glib::ustring parameters_units_value = "angstrom";
			Glib::ustring parameters_units_id = "CELL_PARAMETERS";

			//lattice vectors
			Gtk::Label *label_v1_1 = Gtk::manage(new Gtk::Label("v1(1)"));
			Gtk::Label *label_v1_2 = Gtk::manage(new Gtk::Label("v1(2)"));
			Gtk::Label *label_v1_3 = Gtk::manage(new Gtk::Label("v1(3)"));
			Gtk::Label *label_v2_1 = Gtk::manage(new Gtk::Label("v2(1)"));
			Gtk::Label *label_v2_2 = Gtk::manage(new Gtk::Label("v2(2)"));
			Gtk::Label *label_v2_3 = Gtk::manage(new Gtk::Label("v2(3)"));
			Gtk::Label *label_v3_1 = Gtk::manage(new Gtk::Label("v3(1)"));
			Gtk::Label *label_v3_2 = Gtk::manage(new Gtk::Label("v3(2)"));
			Gtk::Label *label_v3_3 = Gtk::manage(new Gtk::Label("v3(3)"));
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v1_1 = Gtk::Adjustment::create(1.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v1_2 = Gtk::Adjustment::create(0.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v1_3 = Gtk::Adjustment::create(0.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v2_1 = Gtk::Adjustment::create(0.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v2_2 = Gtk::Adjustment::create(1.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v2_3 = Gtk::Adjustment::create(0.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v3_1 = Gtk::Adjustment::create(0.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v3_2 = Gtk::Adjustment::create(0.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Glib::RefPtr<Gtk::Adjustment> adj_sb_v3_3 = Gtk::Adjustment::create(1.0, -100.0, 100.0, 0.000001, 100.0, 0.0);
			Gtk::SpinButton sbutton_v1_1, sbutton_v1_2, sbutton_v1_3, sbutton_v2_1, sbutton_v2_2, sbutton_v2_3, sbutton_v3_1, sbutton_v3_2, sbutton_v3_3;

		//ATOMS tab
			Gtk::Box box_main_atoms, box_positions_units, box_atoms_in_frame, box_atoms_labels;
			Gtk::Frame frame_atoms;
			Gtk::ScrolledWindow scrolled_box_atoms;
			add_atomic_pos_window aap_win;

			//positions_units
			ModelColumns positions_units_columns;
			Gtk::ComboBox positions_units_combobox;
			Glib::RefPtr<Gtk::ListStore> positions_units_list;
			Glib::ustring positions_units_value = "angstrom";
			Glib::ustring positions_units_id = "ATOMIC_POSITIONS";

			//atoms' specifications
			Gtk::Label *label_atom_info = Gtk::manage(new Gtk::Label("Please update ATOMIC_SPECIES before adding positions for atoms to avoid mistakes."));
			Gtk::Label *label_atom_name = Gtk::manage(new Gtk::Label("Chemical symbol:"));
			Gtk::Label *label_atom_mass = Gtk::manage(new Gtk::Label("Mass:"));
			Gtk::Label *label_atom_pspot = Gtk::manage(new Gtk::Label("Pseudopotential file:"));
			Gtk::Label *label_atom_num = Gtk::manage(new Gtk::Label("Number of atoms:"));
			Gtk::Label *label_atom_empty = Gtk::manage(new Gtk::Label(" "));

			vector<Gtk::Entry*> atoms_names_entries;
			vector<Gtk::SpinButton*> atoms_mass_sbuttons;
			vector<Glib::RefPtr<Gtk::Adjustment>> atoms_mass_adj;
			vector<Gtk::Entry*> atoms_pspot_entries;
			vector<Gtk::SpinButton*> atoms_num_sbuttons;
			vector<Glib::RefPtr<Gtk::Adjustment>> atoms_num_adj;
			vector<Gtk::Button> atoms_buttons;
			vector<Gtk::Box*> atoms_boxes;
			Gtk::Button button_update_species;
			int signal_times = 0;
			int option_add_atom; // 0 -- just add new; 1 -- add extra, 2 -- replace

		//SCRIPT tab
		Gtk::TextView script_field;
		Gtk::ScrolledWindow script_scroll;
		Glib::RefPtr<Gtk::TextBuffer> script_buffer;
		Gtk::Button save_script_button;
		Gtk::Button clear_script_button;


};
#endif
